(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('styled-components'), require('react')) :
  typeof define === 'function' && define.amd ? define(['exports', 'styled-components', 'react'], factory) :
  (factory((global.reactGridFlexbox = {}),global.styled,global.React));
}(this, (function (exports,styled,React) { 'use strict';

  styled = styled && styled.hasOwnProperty('default') ? styled['default'] : styled;

  var defaultBreakpoints = {
      tn: '0px', xs: '500px', sm: '768px', md: '992px',
      lg: '1200px', xl: '1440px', hg: '1920px'
  };
  var composeCSSMedia = function composeCSSMedia(cssProp, value, defaultValue) {
      var parseValue = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : function (_value) {
          return _value;
      };

      if (typeof value === 'string') {
          return cssProp + ': ' + parseValue(value) + ';';
      } else if (value === Object(value)) {
          return Object.keys(value).reduce(function (mediaString, key) {
              return '\n      ' + mediaString + '\n      @media (min-width: ' + (defaultBreakpoints[key] ? defaultBreakpoints[key] : key) + ') {\n        ' + cssProp + ': ' + parseValue(value[key]) + ';\n      }\n    ';
          }, '');
      }
      return defaultValue ? cssProp + ': ' + parseValue(defaultValue) + ';' : '';
  };

  var taggedTemplateLiteral = function (strings, raw) {
    return Object.freeze(Object.defineProperties(strings, {
      raw: {
        value: Object.freeze(raw)
      }
    }));
  };

  var _templateObject = taggedTemplateLiteral(['', ''], ['', '']);
  var StyledGridItem = styled('div')(_templateObject, function (p) {
      return '\n    ' + composeCSSMedia('grid-area', p._gridArea, 'auto') + '\n  ';
  });

  var GridItem = function GridItem(props) {
      return React.createElement(React.Fragment, null, React.createElement(StyledGridItem, { _gridArea: props._gridArea }, props.children));
  };

  var _templateObject$1 = taggedTemplateLiteral(['', ''], ['', '']);
  var StyledGridItem$1 = styled('div')(_templateObject$1, function (p) {
      return '\n    display: grid;\n    ' + composeCSSMedia('grid-template-columns', p.templateColumns, 'auto') + '\n    ' + composeCSSMedia('grid-template-rows', p.templateRows, 'auto') + '\n    ' + composeCSSMedia('grid-template-areas', p.templateAreas) + '\n    ' + composeCSSMedia('grid-gap', p.gutter, '0px') + '\n    ' + (p.incGutterEdges ? composeCSSMedia('padding', p.gutter) : '') + '\n  ';
  });

  var keysToDelete = ['_gridArea', 'data-grid-area'];
  var Grid = function Grid(props) {
      var gridChildren = React.Children.map(props.children, function (child) {
          // Wrap any children in a div to prevent potential css flex layout overrides.
          if (child) {
              var type = child.type;
              if (type === GridItem) {
                  return child;
              }
              var wrappedgGridChildProps = {
                  '_gridArea': child.props['data-grid-area'] || child.props._gridArea
              };
              var childPropsWithoutFlexProps = Object.assign({}, child.props);
              keysToDelete.forEach(function (key) {
                  delete childPropsWithoutFlexProps[key];
              });
              var wrapped = React.createElement('div', wrappedgGridChildProps, child.props ? Object.assign({}, child, { props: childPropsWithoutFlexProps }) : child);
              return React.createElement(GridItem, Object.assign({}, wrapped.props));
          }
          return null;
      });
      var jsx = function jsx() {
          return React.createElement(StyledGridItem$1, { templateRows: props.templateRows, templateColumns: props.templateColumns, templateAreas: props.templateAreas, gutter: props.gutter, incGutterEdges: props.incGutterEdges }, gridChildren);
      };
      return props.className ? React.createElement("div", { className: props.className }, jsx()) : jsx();
  };

  var _templateObject$2 = taggedTemplateLiteral(['', ''], ['', '']);
  var StyledFlexItem = styled('div')(_templateObject$2, function (p) {
    return '\n    ' + composeCSSMedia('padding', p.gutterSize, null, function (value) {
      return 'calc(' + value + ' / 2)';
    }) + '\n    ' + (p._flexGrow ? 'flex-grow: 1;' : '') + '\n    ' + (p._flexBasis ? composeCSSMedia('flex-basis', p._flexBasis, 'auto') : '') + '\n    ' + (p._flexGrow ? 'flex-grow: 1;' : '') + '\n    ' + (p._flexScroll ? '\n      > * {\n        ' + composeCSSMedia('max-width', p.gutterSize, '0px', function (value) {
      return 'calc(100% - ' + value + ')';
    }) + '\n        ' + composeCSSMedia('max-heigth', p.gutterSize, '0px', function (value) {
      return 'calc(100% - ' + value + ')';
    }) + '\n        overflow: auto;\n      }\n    ' : '') + '\n    ' + (p._flexReset ? '\n      position: relative;\n      > * {\n        position: absolute;\n        ' + composeCSSMedia('min-width', p.gutterSize, '100%', function (value) {
      return 'calc(100% - ' + value + ')';
    }) + '\n        ' + composeCSSMedia('min-height', p.gutterSize, '100%', function (value) {
      return 'calc(100% - ' + value + ')';
    }) + '\n      }\n    ' : '') + '\n  ';
  });

  var FlexItem = function FlexItem(props) {
      var jsx = function jsx() {
          return props._flexScroll || props._flexReset ? React.createElement("div", null, props.children) : props.children;
      };
      return React.createElement(StyledFlexItem, { gutterSize: props.gutterSize, _flexBasis: props._flexBasis, _flexGrow: props._flexGrow, _flexScroll: props._flexScroll, _flexReset: props._flexReset }, jsx());
  };

  var _templateObject$3 = taggedTemplateLiteral(['', ''], ['', '']);
  var composeAlignment = function composeAlignment(props) {
    var alignMap = {
      top: 'flex-start',
      bottom: 'flex-end',
      left: 'flex-start',
      right: 'flex-end',
      center: 'center',
      middle: 'center'
    };
    return '\n    ' + (props.vAlign ? (props.isRow ? 'align-items' : 'justify-content') + ': ' + alignMap[props.vAlign] + ';' : '') + '\n    ' + (props.hAlign ? (props.isRow ? 'justify-content' : 'align-items') + ': ' + alignMap[props.hAlign] + ';' : '') + '\n  ';
  };
  var StyledFlex = styled('div')(_templateObject$3, function (p) {
    return '\n    display: flex;\n    align-content: flex-start;\n    ' + (p.direction ? 'flex-direction: ' + p.direction + ';' : '') + '\n    flex-wrap: ' + (p.wrap && p.isRow ? 'wrap' : 'nowrap') + ';\n    ' + composeAlignment(p) + '\n    ' + (p.incGutterEdges ? '\n      margin: 0;\n      ' + composeCSSMedia('padding', p.gutter, null, function (value) {
      return 'calc(' + value + ' / 2)';
    }) + '\n    ' : '\n      ' + composeCSSMedia('margin', p.gutter, null, function (value) {
      return 'calc(-' + value + ' / 2)';
    }) + '\n    ') + '\n    ' + (p.full ? '\n      ' + composeCSSMedia('min-width', p.gutter, '0px', function (value) {
      return 'calc(100% + ' + value + ')';
    }) + '\n      ' + composeCSSMedia('min-height', p.gutter, '0px', function (value) {
      return 'calc(100% + ' + value + ')';
    }) + '\n    ' : '') + '\n\n    > * {\n      max-width: 100%;\n    }\n  ';
  });

  var keysToDelete$1 = ['_flexBasis', 'data-flex-basis', '_flexGrow', 'data-flex-grow', '_flexScroll', 'data-flex-scroll', '_flexReset', 'data-flex-reset'];
  var Flex = function Flex(props) {
      var flexChildren = React.Children.map(props.children, function (child) {
          // Wrap any children in a div to prevent potential css flex layout overrides.
          if (child) {
              var type = child.type;
              if (type === FlexItem) {
                  return child;
              }
              var wrappedFlexChildProps = {
                  '_flexBasis': child.props['data-flex-basis'] || child.props._flexBasis,
                  '_flexGrow': child.props['data-flex-grow'] || child.props._flexGrow,
                  '_flexScroll': child.props['data-flex-scroll'] || child.props._flexScroll,
                  '_flexReset': child.props['data-flex-reset'] || child.props._flexReset
              };
              var childPropsWithoutFlexProps = Object.assign({}, child.props);
              keysToDelete$1.forEach(function (key) {
                  delete childPropsWithoutFlexProps[key];
              });
              var wrapped = React.createElement('div', wrappedFlexChildProps, child.props ? Object.assign({}, child, { props: childPropsWithoutFlexProps }) : child);
              return React.createElement(FlexItem, Object.assign({}, wrapped.props, { gutterSize: props.gutter }));
          }
          return null;
      });
      var jsx = function jsx() {
          return React.createElement(StyledFlex, { direction: props.direction ? props.direction : 'column', vAlign: props.vAlign, hAlign: props.hAlign, gutter: props.gutter, full: props.full, wrap: props.wrap, incGutterEdges: props.incGutterEdges }, flexChildren);
      };
      return props.className ? React.createElement("div", { className: props.className }, jsx()) : jsx();
  };

  exports.Grid = Grid;
  exports.GridItem = GridItem;
  exports.Flex = Flex;
  exports.FlexItem = FlexItem;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
